# -*- coding: utf-8 -*-
'''
A lightweight proxy for saltstack
'''

# Import python libs
import logging
import multiprocessing

# Import third party libs
import zmq

# Import salt libs
import salt.minion
import salt.utils

# Import saltbroker utils
from saltbroker.utils import appendproctitle

log = logging.getLogger(__name__)

class PubBroker(multiprocessing.Process):
    '''
    Publish broker, a simple zeromq PUB/SUB proxy
    '''
    def __init__(self, opts):
        super(PubBroker, self).__init__()
        self.opts = opts
        self.opts['master_ip'] = salt.utils.dns_check(self.opts['master'])

    def run(self):
        '''
        Build a PUB/SUB proxy

        http://zguide.zeromq.org/py:wuproxy
        '''
        appendproctitle(self.__class__.__name__)
        context = zmq.Context()

        # Set up a master SUB sock
        master_pub = 'tcp://{0}:{1}'.format(self.opts['master_ip'],
                                            self.opts['publish_port'])
        sub_sock = context.socket(zmq.SUB)
        log.info('Starting set up a broker SUB sock on {0}'.format(master_pub))
        sub_sock.connect(master_pub)

        # Set up a broker PUB sock
        pub_uri = 'tcp://{0}:{1}'.format(self.opts['interface'],
                                         self.opts['publish_port'])
        pub_sock = context.socket(zmq.PUB)
        log.info('Starting set up a broker PUB sock on {0}'.format(pub_uri))
        pub_sock.bind(pub_uri)

        # Subscribe everything from master
        sub_sock.setsockopt(zmq.SUBSCRIBE, b'')

        try:
            # Send  messages to broker PUB sock
            while True:
                message = sub_sock.recv_multipart()
                pub_sock.send_multipart(message)
        except KeyboardInterrupt:
            log.warn('Stopping the PubBorker')
            if sub_sock.closed is False:
                sub_sock.setsockopt(zmq.LINGER, 1)
                sub_sock.close()
            if pub_sock.closed is False:
                pub_sock.setsockopt(zmq.LINGER, 1)
                pub_sock.close()
            if context.closed is False:
                context.term()


class RetBroker(multiprocessing.Process):
    '''
    Ret broker, a simple zeromq message queuing broker
    '''
    def __init__(self, opts):
        super(RetBroker, self).__init__()
        self.opts = opts
        self.opts['master_ip'] = salt.utils.dns_check(self.opts['master'])

    def run(self):
        '''
        Build a QUEUE device

        http://zguide.zeromq.org/py:msgqueue
        '''
        appendproctitle(self.__class__.__name__)
        context = zmq.Context()

        # Set up a router sock to receive results from minions
        router_uri = 'tcp://{0}:{1}'.format(self.opts['interface'],
                                            self.opts['ret_port'])
        router_sock = context.socket(zmq.ROUTER)
        log.info(
            'Starting set up a broker ROUTER sock on {0}'.format(router_uri))
        router_sock.bind(router_uri)

        # Set up a dealer sock to send results to master ret interface
        dealer_sock = context.socket(zmq.DEALER)
        self.opts['master_uri'] = 'tcp://{0}:{1}'.format(self.opts['master_ip'],
                                                      self.opts['ret_port'])
        log.info(
            'Starting set up a broker DEALER sock on {0}'.format(
                self.opts['master_uri']))
        dealer_sock.connect(self.opts['master_uri'])

        try:
            # Forward all results
            zmq.device(zmq.QUEUE, router_sock, dealer_sock)
        except KeyboardInterrupt:
            log.warn('Stopping the RetBroker')
            if router_sock.closed is False:
                router_sock.setsockopt(zmq.LINGER, 1)
                router_sock.close()
            if dealer_sock.closed is False:
                dealer_sock.setsockopt(zmq.LINGER, 1)
                dealer_sock.close()
            if context.closed is False:
                context.term()



class Broker(object):
    '''
    Salt broker
    '''
    def __init__(self, opts):
        '''
        Create a salt broker instance
        '''
        self.opts = opts

    def start(self):
        '''
        Turn on broker components
        '''
        log.info(
            'salt-broker is starting as user {0!r}'.format(
                salt.utils.get_user())
        )
        log.info('starting pub broker......')
        self.pub_broker = PubBroker(self.opts)
        self.pub_broker.start()
        log.info('starting ret broker......')
        self.ret_broker = RetBroker(self.opts)
        self.ret_broker.start()




# -*- coding: utf-8 -*-

# Import python libs

# Import salt libs
import salt.log.setup
from salt.utils.verify import check_user

import saltbroker.broker
from saltbroker.utils import parsers

logger = salt.log.setup.logging.getLogger(__name__)


class Broker(parsers.BrokerOptionParser):
    '''
    Create a broker server
    '''
    def prepare(self):
        self.parse_args()

        self.setup_logfile_logger()
        logger.info('Setting up Salt Broker')

        self.daemonize_if_required()
        self.set_pidfile()
        self.broker = saltbroker.broker.Broker(self.config)

    def start(self):
        '''
        Start broker.
        '''
        self.prepare()
        if check_user(self.config['user']):
            self.broker.start()

    def shutdown(self):
        '''
        Shutdown
        '''



# -*- coding: utf-8 -*-

# Import python libs
import os

# Import salt libs
import salt
from salt.config import load_config

_DFLT_LOG_DATEFMT = '%H:%M:%S'
_DFLT_LOG_DATEFMT_LOGFILE = '%Y-%m-%d %H:%M:%S'
_DFLT_LOG_FMT_CONSOLE = '[%(levelname)-8s] %(message)s'
_DFLT_LOG_FMT_LOGFILE = (
    '%(asctime)s,%(msecs)03.0f [%(name)-17s][%(levelname)-8s] %(message)s'
)

DEFAULT_BROKER_OPTS = {
    'interface': '0.0.0.0',
    'master': 'salt',
    'ret_port': '4506',
    'publish_port': '4505',
    'ipv6': False,
    'user': 'root',
    'conf_file': os.path.join(salt.syspaths.CONFIG_DIR, 'broker'),
    'log_file': os.path.join(salt.syspaths.LOGS_DIR, 'broker'),
    'log_level': None,
    'log_datefmt': _DFLT_LOG_DATEFMT,
    'log_datefmt_logfile': _DFLT_LOG_DATEFMT_LOGFILE,
    'log_fmt_console': _DFLT_LOG_FMT_CONSOLE,
    'log_fmt_logfile': _DFLT_LOG_FMT_LOGFILE,
    'log_granular_levels': {},
    'extension_modules': '',
    'pidfile': os.path.join(salt.syspaths.PIDFILE_DIR, 'salt-broker.pid'),
}

def broker_config(path, env_var='SALT_BROKER_CONFIG', defaults=None):
    '''
    Reads in the broker configuration file and sets up default options
    '''
    if defaults is None:
        defaults = DEFAULT_BROKER_OPTS

    opts = {}
    opts.update(DEFAULT_BROKER_OPTS)
    if not os.environ.get(env_var, None):
        salt_config_dir = os.environ.get('SALT_CONFIG_DIR', None)
        if salt_config_dir:
            env_config_file_path = os.path.join(salt_config_dir, 'broker')
            if os.path.isfile(env_config_file_path):
                os.environ[env_var] = env_config_file_path

    opts.update(load_config(path, env_var, DEFAULT_BROKER_OPTS['conf_file']))
    return opts



# -*- coding: utf-8 -*-
'''
Some of the utils used by salt broker
'''

try:
    import setproctitle
    HAS_SETPROCTITLE = True
except ImportError:
    HAS_SETPROCTITLE = False


def appendproctitle(name):
    '''
    Append "name" to the current process title

    From: https://github.com/saltstack/salt/blob/v2014.7.1/salt/utils/__init__.py#L2377
    '''
    if HAS_SETPROCTITLE:
        setproctitle.setproctitle(setproctitle.getproctitle() + ' ' + name)



# -*- coding: utf-8 -*-
NAME = 'salt-broker'
DESCRIPTION = 'A lightweight proxy for saltstack'
VERSION = '0.2.3'
AUTHOR = 'pengyao'
AUTHOR_EMAIL = 'pengyao@pengyao.org'
COPYRIGHT = 'Copyright © 2015 ' + AUTHOR
LICENCE = 'Apache License V2.0'
URL = 'https://github.com/pengyao/salt-broker'






# -*- coding: utf-8 -*-
'''
This module contains the function calls to execute command line scipts
'''

# Import salt libs
import saltbroker

def salt_broker():
    '''
    Start the salt broker
    '''
    broker = saltbroker.Broker()
    broker.start()


# -*- coding: utf-8 -*-

# Import python libs
import os

# Import salt libs
from salt.utils.parsers import OptionParserMeta, OptionParser, ConfigDirMixIn, \
    LogLevelMixIn, RunUserMixin, DaemonMixIn, PidfileMixin
import salt.syspaths as syspaths

import saltbroker.config as config

class BrokerOptionParser(OptionParser, ConfigDirMixIn, LogLevelMixIn,
    RunUserMixin, DaemonMixIn, PidfileMixin):

    __metaclass__ = OptionParserMeta

    description = 'The Salt broker, used to forward message between master and minions.'

    # ConfigDirMixIn config filename attribute
    _config_filename_ = 'broker'
    # LogLevelMixIn attribute
    _default_logging_logfile_ = os.path.join(syspaths.LOGS_DIR, 'broker')

    def setup_config(self):
        return config.broker_config(self.get_config_file_path())



#!/usr/bin/env python

from saltbroker.metadata import *
from setuptools import setup, find_packages

REQUIREMENTS = [
    'pyzmq>=2.2.0',
    'salt>=2014.1.0',
]

setup(
    name=NAME,
    description=DESCRIPTION,
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENCE,
    url=URL,
    packages=find_packages(exclude=['docs']),
    install_requires=REQUIREMENTS,
    scripts=['scripts/salt-broker']
)